<?php

/**
*  function hook_votingapi_field_form_generators()
*  This hook can be used to provide custom input forms for the 'Input Form'.
*  This hook should return an array of form infos. A form info is an
*  associative array with 4 keys: 'name' is the display name of the form info,
*  'form info' is the form form info function name, 'js' true or false: does
*  this field require javascript, and finally 'type' is an array with one or
*  more of the following strings in it: 'choice', 'percent', 'score'.
*/
function votingapi_field_votingapi_field_form_generators() {
  $form_infos = array();
  $form_infos[] = array(
    'name' => t('Choice menu'),
    'generator' => 'votingapi_field_std_choice',
    'js' => false,
    'types' => array('votingapi_choice')
  );
  $form_infos[] = array(
    'name' => t('Choice menu 5% increments'),
    'generator' => 'votingapi_field_std_percent',
    'js' => false,
    'types' => array('votingapi_percent')
  );
  $form_infos[] = array(
    'name' => t('Choice menu'),
    'generator' => 'votingapi_field_std_score',
    'js' => false,
    'types' => array('votingapi_score')
  );
  return $form_infos;
}

/**
* get the options for a rating type field
*/
function votingapi_field_get_rating_options($field) {
  $options = array();
  $min = intval($field['minval']);
  $max = intval($field['maxval']);
  for ($i=$min; $i<=$max; $i++) $options[$i] = $i;
  return $options;
}

/**
* get the options for a choice type field
*/
function votingapi_field_get_choice_options($field) {
  // these options are bad - if the labels are modified the numbers will no
  // longer jive. we will probably have to store a map of indices to labels
  $options = array();
  $labels = split("\r\n", $field['labels']);
  $idx = 0;
  foreach ($labels as $label) {
    $options[$idx] = $label;
    $idx++;
  }
  return $options;
}

/**
* choice form
*/
function votingapi_field_vote_form_votingapi_choice($node, $field) {
  $form = array();
  $options = votingapi_field_get_choice_options($field);
  $form['vote'] = array(
    '#title' => t($field['widget']['label']),
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => '',
    '#required' => $field['required'],
  );
  $form['nid'] = array(
    '#type' => 'hidden',
    '#value' => $node->nid,
  );
  $form['destination'] = array(
    '#type' => 'hidden',
    '#value' => $_GET['q'],
  );
  $form['field_name'] = array(
    '#type' => 'hidden',
    '#value' => $field['field_name'],
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Vote'),
  );
  return $form;
}

/*
* validate a choice vote
*/
function votingapi_field_vote_form_votingapi_choice_validate($form_id, $edit) {
  $field = content_fields($edit['field_name']);
  $voteval = intval($edit['vote']);
  $labels = split("\r\n", $field['labels']);
  $maxval = count($labels)-1;
  if ($voteval < 0) form_set_error('vote', t("Negative vote value."));
  if ($voteval > $maxval) form_set_error(
    'vote', t("Vote value exceeds max %d.", array('%d'=>$maxval))
  );
}

/*
* called when a user votes on a choice field
*/
function votingapi_field_vote_form_votingapi_choice_submit($form_id, $edit) {
  _votingapi_field_submit_vote($form_id, $edit, 'option');
}

/**
* get the options for a score type field
*/
function votingapi_field_get_score_options($field) {
  // these options are bad - if the labels are modified the numbers will no
  // longer jive. we will probably have to store a map of indices to labels
  $options = array();
  if ($field['labels']) {
    $labels = split("\r\n", $field['labels']);
    $idx = intval($field['minval']);
     foreach ($labels as $label) {
      $options[$idx] = $label;
      $idx++;
    }
  }
  else {
    // no labels, just display the value
    $min = intval($field['minval']);
    $max = intval($field['maxval']);
    for ($i=$min; $i<=$max; $i++) $options[$i] = $i;
  }
  return $options;
}

/**
* score form
*/
function votingapi_field_std_score($node, $field) {

  $form = array();

  $form['nid'] = array(
    '#type' => 'hidden',
    '#value' => $node->nid,
  );
  $form['destination'] = array(
    '#type' => 'hidden',
    '#value' => $_GET['q'],
  );
  $form['field_name'] = array(
    '#type' => 'hidden',
    '#value' => $field['field_name'],
  );

  $minval = intval($field['minval']);
  $maxval = intval($field['maxval']);
  
  $options = votingapi_field_get_score_options($field);
  if ($minval==$maxval) {
    $form['vote'] = array(
      '#type' => 'hidden',
      '#value' => $minval,
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => theme('votingapi_field_single_vote_button', $field),
    );
  }
  else {
    $form['vote'] = array(
      '#title' => t($field['widget']['label']),
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => '',
      '#required' => $field['required'],
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Vote'),
    );
  }
  return $form;
}

/*
* validate a score vote
*/
function votingapi_field_std_score_validate($form_id, $edit) {
  $field = content_fields($edit['field_name']);
  $voteval = intval($edit['vote']);
  $labels = split("\r\n", $field['labels']);
  $minval = intval($field['minval']);
  $maxval = intval($field['maxval']);
  if ($voteval < $minval) form_set_error(
    'vote', t("Vote value exceeds min %d.", array('%d'=>$minval))
  );
  if ($voteval > $maxval) form_set_error(
    'vote', t("Vote value exceeds max %d.", array('%d'=>$maxval))
  );
}

/*
* called when a user votes on a score field
*/
function votingapi_field_std_score_submit($form_id, $edit) {
  _votingapi_field_submit_vote($form_id, $edit, 'points');
}

/**
* get options for a percent type field
*/
function votingapi_field_get_percent_options($field) {
  return array(
    '0'   => '0%',
    '5'   => '5%',
    '10'  => '10%',
    '15'  => '15%',
    '20'  => '20%',
    '25'  => '25%',
    '30'  => '30%',
    '35'  => '35%',
    '40'  => '40%',
    '45'  => '45%',
    '50'  => '50%',
    '55'  => '55%',
    '60'  => '60%',
    '65'  => '65%',
    '70'  => '70%',
    '75'  => '75%',
    '80'  => '80%',
    '85'  => '85%',
    '90'  => '90%',
    '95'  => '95%',
    '100' => '100%',
  );
}

/**
* percent form
*/
function votingapi_field_std_percent($node, $field) {
  $form = array();

  $form['nid'] = array(
    '#type' => 'hidden',
    '#value' => $node->nid,
  );
  $form['destination'] = array(
    '#type' => 'hidden',
    '#value' => $_GET['q'],
  );
  $form['field_name'] = array(
    '#type' => 'hidden',
    '#value' => $field['field_name'],
  );

  $options = votingapi_field_get_percent_options($field);
  $form['vote'] = array(
    '#title' => t($field['widget']['label']),
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => '',
    '#required' => $field['required'],
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Vote'),
  );
  return $form;
}

/*
* validate a percent vote
*/
function votingapi_field_std_percent_validate($form_id, $edit) { 
  $field = content_fields($edit['field_name']);
  $voteval = intval($edit['vote']);
  if ($voteval < 0) form_set_error('vote', t("Vote value lower than 0%."));
  if ($voteval > 100) form_set_error('vote', t("Vote value exceeds 100%."));
}

/*
* called when a user votes on a percent field
*/
function votingapi_field_std_percent_submit($form_id, $edit) {
  _votingapi_field_submit_vote($form_id, $edit, 'percent');
}
